@@ -1,4 +1,4 @@
-/* $Id: buffer.c,v 1.13 2002/03/15 19:02:40 ukai Exp $ */
+/* $Id: buffer.c,v 1.27 2003/02/03 15:49:19 ukai Exp $ */
 #include "fm.h"
 
 #ifdef USE_MOUSE
@@ -32,6 +32,7 @@ newBuffer(int width)
     bzero((void *)n, sizeof(Buffer));
     n->width = width;
     n->COLS = COLS;
+    n->LINES = LASTLINE;
     n->currentURL.scheme = SCM_UNKNOWN;
     n->baseURL = NULL;
     n->baseTarget = NULL;
@@ -39,7 +40,6 @@ newBuffer(int width)
     n->bufferprop = BP_NORMAL;
     n->clone = New(int);
     *n->clone = 1;
-    n->linelen = 0;
     n->trbyte = 0;
 #ifdef USE_SSL
     n->ssl_certificate = NULL;
@@ -96,10 +96,13 @@ discardBuffer(Buffer *buf)
 	return;
     if (buf->pagerSource)
 	ISclose(buf->pagerSource);
-    if (buf->sourcefile) {
+    if (buf->sourcefile &&
+	(!buf->real_type || strncasecmp(buf->real_type, "image/", 6))) {
 	if (buf->real_scheme != SCM_LOCAL || buf->bufferprop & BP_FRAME)
 	    unlink(buf->sourcefile);
     }
+    if (buf->header_source)
+	unlink(buf->header_source);
     if (buf->mailcap_source)
 	unlink(buf->mailcap_source);
     while (buf->frameset) {
@@ -252,15 +255,16 @@ gotoLine(Buffer *buf, int n)
 	sprintf(msg, "Last line is #%ld", buf->lastLine->linenumber);
 	set_delayed_message(msg);
 	buf->currentLine = l;
-	buf->topLine = lineSkip(buf, buf->currentLine, -(LASTLINE - 1), FALSE);
+	buf->topLine = lineSkip(buf, buf->currentLine, -(buf->LINES - 1),
+				FALSE);
 	return;
     }
     for (; l != NULL; l = l->next) {
 	if (l->linenumber >= n) {
 	    buf->currentLine = l;
 	    if (n < buf->topLine->linenumber ||
-		buf->topLine->linenumber + LASTLINE <= n)
-		buf->topLine = lineSkip(buf, l, -(LASTLINE + 1) / 2, FALSE);
+		buf->topLine->linenumber + buf->LINES <= n)
+		buf->topLine = lineSkip(buf, l, -(buf->LINES + 1) / 2, FALSE);
 	    break;
 	}
     }
@@ -294,15 +298,16 @@ gotoRealLine(Buffer *buf, int n)
 	sprintf(msg, "Last line is #%ld", buf->lastLine->real_linenumber);
 	set_delayed_message(msg);
 	buf->currentLine = l;
-	buf->topLine = lineSkip(buf, buf->currentLine, -(LASTLINE - 1), FALSE);
+	buf->topLine = lineSkip(buf, buf->currentLine, -(buf->LINES - 1),
+				FALSE);
 	return;
     }
     for (; l != NULL; l = l->next) {
 	if (l->real_linenumber >= n) {
 	    buf->currentLine = l;
 	    if (n < buf->topLine->real_linenumber ||
-		buf->topLine->real_linenumber + LASTLINE <= n)
-		buf->topLine = lineSkip(buf, l, -(LASTLINE + 1) / 2, FALSE);
+		buf->topLine->real_linenumber + buf->LINES <= n)
+		buf->topLine = lineSkip(buf, l, -(buf->LINES + 1) / 2, FALSE);
 	    break;
 	}
     }
@@ -495,6 +500,7 @@ reshapeBuffer(Buffer *buf)
     if (!buf->need_reshape)
 	return;
     buf->need_reshape = FALSE;
+    buf->width = INIT_BUFFER_WIDTH;
     if (buf->sourcefile == NULL)
 	return;
     init_stream(&f, SCM_LOCAL, NULL);
@@ -513,15 +519,17 @@ reshapeBuffer(Buffer *buf)
     buf->name = NULL;
     buf->img = NULL;
     buf->formitem = NULL;
-    buf->width = INIT_BUFFER_WIDTH;
+    buf->formlist = NULL;
+    buf->linklist = NULL;
+    buf->maplist = NULL;
+    if (buf->hmarklist)
+	buf->hmarklist->nmark = 0;
+    if (buf->imarklist)
+	buf->imarklist->nmark = 0;
 
-#ifdef JP_CHARSET
-    UseContentCharset = FALSE;
-    UseAutoDetect = FALSE;
-#endif
-    if (buf->search_header && buf->currentURL.scheme == SCM_LOCAL) {
-	if (buf->header_source && (buf->mailcap_source ||
-				   !strcmp(buf->currentURL.file, "-"))) {
+    if (buf->header_source) {
+	if (buf->currentURL.scheme != SCM_LOCAL ||
+	    buf->mailcap_source || !strcmp(buf->currentURL.file, "-")) {
 	    URLFile h;
 	    init_stream(&h, SCM_LOCAL, NULL);
 	    examineFile(buf->header_source, &h);
@@ -530,10 +538,14 @@ reshapeBuffer(Buffer *buf)
 		UFclose(&h);
 	    }
 	}
-	else
+	else if (buf->search_header)	/* -m option */
 	    readHeader(&f, buf, TRUE, NULL);
     }
 
+#ifdef JP_CHARSET
+    UseContentCharset = FALSE;
+    UseAutoDetect = FALSE;
+#endif
     if (!strcasecmp(buf->type, "text/html"))
 	loadHTMLBuffer(&f, buf);
     else
@@ -545,13 +557,40 @@ reshapeBuffer(Buffer *buf)
 #endif
 
     buf->height = LASTLINE + 1;
-    if (buf->firstLine)
-	restorePosition(buf, &sbuf);
+    if (buf->firstLine && sbuf.firstLine) {
+	Line *cur = sbuf.currentLine;
+	int n;
+
+	buf->pos = sbuf.pos + cur->bpos;
+	while (cur->bpos && cur->prev)
+	    cur = cur->prev;
+	if (cur->real_linenumber > 0)
+	    gotoRealLine(buf, cur->real_linenumber);
+	else
+	    gotoLine(buf, cur->linenumber);
+	n = (buf->currentLine->linenumber - buf->topLine->linenumber)
+	    - (cur->linenumber - sbuf.topLine->linenumber);
+	if (n) {
+	    buf->topLine = lineSkip(buf, buf->topLine, n, FALSE);
+	    if (cur->real_linenumber > 0)
+		gotoRealLine(buf, cur->real_linenumber);
+	    else
+		gotoLine(buf, cur->linenumber);
+	}
+	buf->pos -= buf->currentLine->bpos;
+	if (FoldLine && strcasecmp(buf->type, "text/html"))
+	    buf->currentColumn = 0;
+	else
+	    buf->currentColumn = sbuf.currentColumn;
+	arrangeCursor(buf);
+    }
     if (buf->check_url & CHK_URL)
 	chkURLBuffer(buf);
 #ifdef USE_NNTP
     if (buf->check_url & CHK_NMID)
 	chkNMIDBuffer(buf);
+    if (buf->real_scheme == SCM_NNTP || buf->real_scheme == SCM_NEWS)
+	reAnchorNewsheader(buf);
 #endif
     formResetBuffer(buf, sbuf.formitem);
 }
@@ -607,16 +646,24 @@ writeBufferCache(Buffer *buf)
 	    fwrite1(l->usrflags, cache) ||
 	    fwrite1(l->width, cache) ||
 	    fwrite1(l->len, cache) ||
-	    fwrite(l->lineBuf, 1, l->len, cache) < l->len ||
-	    fwrite(l->propBuf, sizeof(Lineprop), l->len, cache) < l->len)
+	    fwrite1(l->size, cache) ||
+	    fwrite1(l->bpos, cache) || fwrite1(l->bwidth, cache))
 	    goto _error;
+	if (l->bpos == 0) {
+	    if (fwrite(l->lineBuf, 1, l->size, cache) < l->size ||
+		fwrite(l->propBuf, sizeof(Lineprop), l->size, cache) < l->size)
+		goto _error;
+	}
 #ifdef USE_ANSI_COLOR
 	colorflag = l->colorBuf ? 1 : 0;
 	if (fwrite1(colorflag, cache))
 	    goto _error;
 	if (colorflag) {
-	    if (fwrite(l->colorBuf, sizeof(Linecolor), l->len, cache) < l->len)
-		goto _error;
+	    if (l->bpos == 0) {
+		if (fwrite(l->colorBuf, sizeof(Linecolor), l->size, cache) <
+		    l->size)
+		    goto _error;
+	    }
 	}
 #endif
     }
@@ -635,7 +682,7 @@ int
 readBufferCache(Buffer *buf)
 {
     FILE *cache;
-    Line *l = NULL, *prevl = NULL;
+    Line *l = NULL, *prevl = NULL, *basel = NULL;
     long lnum = 0, clnum, tlnum;
 #ifdef USE_ANSI_COLOR
     int colorflag;
@@ -666,19 +713,35 @@ readBufferCache(Buffer *buf)
 	    buf->topLine = l;
 	if (fread1(l->real_linenumber, cache) ||
 	    fread1(l->usrflags, cache) ||
-	    fread1(l->width, cache) || fread1(l->len, cache))
+	    fread1(l->width, cache) ||
+	    fread1(l->len, cache) ||
+	    fread1(l->size, cache) ||
+	    fread1(l->bpos, cache) || fread1(l->bwidth, cache))
+	    break;
+	if (l->bpos == 0) {
+	    basel = l;
+	    l->lineBuf = NewAtom_N(char, l->size + 1);
+	    fread(l->lineBuf, 1, l->size, cache);
+	    l->lineBuf[l->size] = '\0';
+	    l->propBuf = NewAtom_N(Lineprop, l->size);
+	    fread(l->propBuf, sizeof(Lineprop), l->size, cache);
+	}
+	else if (basel) {
+	    l->lineBuf = basel->lineBuf + l->bpos;
+	    l->propBuf = basel->propBuf + l->bpos;
+	}
+	else
 	    break;
-	l->lineBuf = NewAtom_N(char, l->len + 1);
-	fread(l->lineBuf, 1, l->len, cache);
-	l->lineBuf[l->len] = '\0';
-	l->propBuf = NewAtom_N(Lineprop, l->len);
-	fread(l->propBuf, sizeof(Lineprop), l->len, cache);
 #ifdef USE_ANSI_COLOR
 	if (fread1(colorflag, cache))
 	    break;
 	if (colorflag) {
-	    l->colorBuf = NewAtom_N(Linecolor, l->len);
-	    fread(l->colorBuf, sizeof(Linecolor), l->len, cache);
+	    if (l->bpos == 0) {
+		l->colorBuf = NewAtom_N(Linecolor, l->size);
+		fread(l->colorBuf, sizeof(Linecolor), l->size, cache);
+	    }
+	    else
+		l->colorBuf = basel->colorBuf + l->bpos;
 	}
 	else {
 	    l->colorBuf = NULL;
