@@ -1,4 +1,4 @@
-/* $Id: etc.c,v 1.67 2003/02/26 17:22:02 ukai Exp $ */
+/* $Id: etc.c,v 1.71 2003/04/06 16:27:54 ukai Exp $ */
 #include "fm.h"
 #include <pwd.h>
 #include "myctype.h"
@@ -84,9 +84,7 @@ lineSkip(Buffer *buf, Line *line, int offset, int last)
     Line *l;
 
     l = currentLineSkip(buf, line, offset, last);
-#ifdef NEXTPAGE_TOPLINE
     if (!nextpage_topline)
-#endif
 	for (i = buf->LINES - 1 - (buf->lastLine->linenumber - l->linenumber);
 	     i > 0 && l->prev != NULL; i--, l = l->prev) ;
     return l;
@@ -1552,7 +1550,8 @@ expandName(char *name)
 	return NULL;
     p = name;
     if (*p == '/') {
-	if (*(p + 1) == '~' && IS_ALPHA(*(p + 2)) && personal_document_root) {
+	if ((*(p + 1) == '~' && IS_ALPHA(*(p + 2)))
+	    && personal_document_root) {
 	    char *q;
 	    p += 2;
 	    q = strchr(p, '/');
