@@ -1,4 +1,4 @@
-/* $Id: fm.h,v 1.113 2003/02/18 15:43:24 ukai Exp $ */
+/* $Id: fm.h,v 1.119 2003/09/22 17:37:41 ukai Exp $ */
 /* 
  * w3m: WWW wo Miru utility
  * 
@@ -32,11 +32,12 @@
 
 #ifndef USE_COLOR
 #undef USE_ANSI_COLOR
+#undef USE_BG_COLOR
 #endif
 
 #include "ctrlcode.h"
 #include "html.h"
-#include "gc.h"
+#include <gc.h>
 #include "Str.h"
 #include "form.h"
 #include "frame.h"
@@ -392,9 +393,9 @@ typedef struct {
 #define LINK_TYPE_REV  2
 typedef struct _LinkList {
     char *url;
-    char *title;	/* Next, Contents, ... */
-    char *ctype;	/* Content-Type */
-    char type;		/* Rel, Rev */
+    char *title;		/* Next, Contents, ... */
+    char *ctype;		/* Content-Type */
+    char type;			/* Rel, Rev */
     struct _LinkList *next;
 } LinkList;
 
@@ -823,8 +824,10 @@ global int NOproxy_netaddr init(TRUE);
 #define DNS_ORDER_UNSPEC     0
 #define DNS_ORDER_INET_INET6 1
 #define DNS_ORDER_INET6_INET 2
+#define DNS_ORDER_INET_ONLY  4
+#define DNS_ORDER_INET6_ONLY 6
 global int DNS_order init(DNS_ORDER_UNSPEC);
-extern int ai_family_order_table[3][3];	/* XXX */
+extern int ai_family_order_table[7][3];	/* XXX */
 #endif				/* INET6 */
 global TextList *NO_proxy_domains;
 global char NoCache init(FALSE);
@@ -845,9 +848,9 @@ global char *index_file init(NULL);
 global char *CurrentDir;
 global int CurrentPid;
 /*
-global Buffer *Currentbuf;
-global Buffer *Firstbuf;
-*/
+ * global Buffer *Currentbuf;
+ * global Buffer *Firstbuf;
+ */
 global TabBuffer *CurrentTab;
 global TabBuffer *FirstTab;
 global TabBuffer *LastTab;
@@ -898,20 +901,12 @@ global int visited_color init(5);	/* magenta  */
 #endif				/* USE_COLOR */
 global int confirm_on_quit init(TRUE);
 #ifdef USE_MARK
-global int use_mark init(TRUE);
+global int use_mark init(FALSE);
 #endif
-#ifdef EMACS_LIKE_LINEEDIT
-global int emacs_like_lineedit init(TRUE);
-#endif
-#ifdef VI_PREC_NUM
-global int vi_prec_num init(TRUE);
-#endif
-#ifdef LABEL_TOPLINE
+global int emacs_like_lineedit init(FALSE);
+global int vi_prec_num init(FALSE);
 global int label_topline init(FALSE);
-#endif
-#ifdef NEXTPAGE_TOPLINE
 global int nextpage_topline init(FALSE);
-#endif
 global char *displayTitleTerm init(NULL);
 global int displayLink init(FALSE);
 global int displayLineInfo init(FALSE);
@@ -928,7 +923,7 @@ global int useExtImageViewer init(TRUE);
 global int maxLoadImage init(4);
 global int image_map_list init(TRUE);
 #else
-global int displayImage init(FALSE); /* XXX: emacs-w3m use display_image=off */
+global int displayImage init(FALSE);	/* XXX: emacs-w3m use display_image=off */
 #endif
 global char *Editor init(DEF_EDITOR);
 #ifdef USE_W3MMAILER
@@ -944,9 +939,7 @@ global int disable_secret_security_check init(FALSE);
 global char *passwd_file init(PASSWD_FILE);
 global char *pre_form_file init(PRE_FORM_FILE);
 global char *ftppasswd init(NULL);
-#ifdef FTPPASS_HOSTNAMEGEN
 global int ftppass_hostnamegen init(TRUE);
-#endif
 global int do_download init(FALSE);
 #ifdef USE_IMAGE
 global char *image_source init(NULL);
@@ -986,7 +979,6 @@ global char *migemo_command init(DEF_MIGEMO_COMMAND);
 #endif				/* USE_MIGEMO */
 
 global struct auth_cookie *Auth_cookie init(NULL);
-global Str Local_cookie init(NULL);
 #ifdef USE_COOKIE
 global struct cookie *First_cookie init(NULL);
 #endif				/* USE_COOKIE */
@@ -1005,6 +997,7 @@ extern Hist *URLHist;
 extern Hist *ShellHist;
 extern Hist *TextHist;
 #ifdef USE_HISTORY
+global int UseHistory init(TRUE);
 global int URLHistSize init(100);
 global int SaveURLHist init(TRUE);
 #endif				/* USE_HISTORY */
