@@ -1,7 +1,7 @@
-/* $Id: indep.h,v 1.13 2003/02/05 16:43:57 ukai Exp $ */
+/* $Id: indep.h,v 1.15 2003/07/26 17:16:24 ukai Exp $ */
 #ifndef INDEP_H
 #define INDEP_H
-#include "gc.h"
+#include <gc.h>
 #include "Str.h"
 #include "config.h"
 
@@ -68,6 +68,7 @@ extern char *shell_quote(char *str);
 extern char *w3m_auxbin_dir();
 extern char *w3m_lib_dir();
 extern char *w3m_etc_dir();
+extern char *w3m_conf_dir();
 extern char *w3m_help_dir();
 
 #define New(type)	((type*)GC_MALLOC(sizeof(type)))
