@@ -1,4 +1,4 @@
-/* $Id: rc.c,v 1.82 2003/02/05 16:48:09 ukai Exp $ */
+/* $Id: rc.c,v 1.86 2003/09/22 17:37:41 ukai Exp $ */
 /* 
  * Initialization file etc.
  */
@@ -57,6 +57,7 @@ static int RC_table_size;
 #define CMT_PIXEL_PER_CHAR      "ʸ (4.0...32.0)"
 #define CMT_PIXEL_PER_LINE      "Ԥι⤵ (4.0...64.0)"
 #define CMT_PAGERLINE    "ڡȤѤ¸Կ"
+#define CMT_HISTORY	 "Ȥ"
 #define CMT_HISTSIZE     "ݻURLο"
 #define CMT_SAVEHIST     "URL¸"
 #define CMT_KANJICODE    "ɽѴ"
@@ -119,18 +120,10 @@ static int RC_table_size;
 #ifdef USE_MARK
 #define CMT_USE_MARK	"ޡǽͭˤ"
 #endif
-#ifdef EMACS_LIKE_LINEEDIT
 #define CMT_EMACS_LIKE_LINEEDIT	"EmacsιԽˤ"
-#endif
-#ifdef VI_PREC_NUM
 #define CMT_VI_PREC_NUM "viοͥץե"
-#endif
-#ifdef LABEL_TOPLINE
 #define CMT_LABEL_TOPLINE	"٥˰ư˥뤬ȥåפˤʤ褦ˤ"
-#endif
-#ifdef NEXTPAGE_TOPLINE
 #define CMT_NEXTPAGE_TOPLINE	"Υڡ˰ư˥뤬ȥåפˤʤ褦ˤ"
-#endif
 #define CMT_FOLD_LINE    "plain text եιԤޤ֤"
 #define CMT_SHOW_NUM     "ֹɽ"
 #define CMT_SHOW_SRCH_STR "ʸɽ"
@@ -146,9 +139,7 @@ static int RC_table_size;
 #define CMT_PASSWDFILE	 "ѥɥե"
 #define CMT_PRE_FORM_FILE	"ʸɹΥեѥե"
 #define CMT_FTPPASS      "FTPΥѥ(̤ϼʬmail addressȤ)"
-#ifdef FTPPASS_HOSTNAMEGEN
 #define CMT_FTPPASS_HOSTNAMEGEN	"FTPΥѥɤΥɥᥤ̾ư"
-#endif
 #define CMT_USERAGENT    "User-Agent"
 #define CMT_ACCEPTENCODING	"ˡ(Accept-Encoding:)"
 #define CMT_ACCEPTMEDIA	 "Ĥǥ(Accept:)"
@@ -222,6 +213,7 @@ static int RC_table_size;
 #define CMT_PIXEL_PER_CHAR      "Number of pixels per character (4.0...32.0)"
 #define CMT_PIXEL_PER_LINE      "Number of pixels per line (4.0...64.0)"
 #define CMT_PAGERLINE    "Number of remembered lines when used as a pager"
+#define CMT_HISTORY	 "Use URL history"
 #define CMT_HISTSIZE     "Number of remembered URL"
 #define CMT_SAVEHIST     "Save URL history"
 /* #define CMT_KANJICODE    "Display Kanji Code" */
@@ -284,18 +276,10 @@ static int RC_table_size;
 #ifdef USE_MARK
 #define CMT_USE_MARK	"Enable mark operations"
 #endif
-#ifdef EMACS_LIKE_LINEEDIT
 #define CMT_EMACS_LIKE_LINEEDIT	"Enable Emacs-style line editing"
-#endif
-#ifdef VI_PREC_NUM
 #define CMT_VI_PREC_NUM	 "Enable vi-like numeric prefix"
-#endif
-#ifdef LABEL_TOPLINE
 #define CMT_LABEL_TOPLINE	"Move cursor to top line when going to label"
-#endif
-#ifdef NEXTPAGE_TOPLINE
 #define CMT_NEXTPAGE_TOPLINE	"Move cursor to top line when moving to next page"
-#endif
 #define CMT_FOLD_LINE    "Fold lines of plain text file"
 #define CMT_SHOW_NUM     "Show line numbers"
 #define CMT_SHOW_SRCH_STR "Show search string"
@@ -311,9 +295,7 @@ static int RC_table_size;
 #define CMT_PASSWDFILE	 "Password file"
 #define CMT_PRE_FORM_FILE	"File for setting form on loading"
 #define CMT_FTPPASS      "Password for anonymous FTP (your mail address)"
-#ifdef FTPPASS_HOSTNAMEGEN
 #define CMT_FTPPASS_HOSTNAMEGEN "Generate domain part of password for FTP"
-#endif
 #define CMT_USERAGENT    "User-Agent identification string"
 #define CMT_ACCEPTENCODING	"Accept-Encoding header"
 #define CMT_ACCEPTMEDIA	 "Accept header"
@@ -488,6 +470,8 @@ static struct sel_c dnsorders[] = {
     {N_S(DNS_ORDER_UNSPEC), "unspecified"},
     {N_S(DNS_ORDER_INET_INET6), "inet inet6"},
     {N_S(DNS_ORDER_INET6_INET), "inet6 inet"},
+    {N_S(DNS_ORDER_INET_ONLY), "inet only"},
+    {N_S(DNS_ORDER_INET6_ONLY), "inet6 only"},
     {0, NULL, NULL}
 };
 #endif				/* INET6 */
@@ -575,14 +559,10 @@ struct param_ptr params1[] = {
     {"show_lnum", P_INT, PI_ONOFF, (void *)&showLineNum, CMT_SHOW_NUM, NULL},
     {"show_srch_str", P_INT, PI_ONOFF, (void *)&show_srch_str,
      CMT_SHOW_SRCH_STR, NULL},
-#ifdef LABEL_TOPLINE
     {"label_topline", P_INT, PI_ONOFF, (void *)&label_topline,
      CMT_LABEL_TOPLINE, NULL},
-#endif
-#ifdef NEXTPAGE_TOPLINE
     {"nextpage_topline", P_INT, PI_ONOFF, (void *)&nextpage_topline,
      CMT_NEXTPAGE_TOPLINE, NULL},
-#endif
     {NULL, 0, 0, NULL, NULL, NULL},
 };
 
@@ -618,6 +598,7 @@ struct param_ptr params2[] = {
 struct param_ptr params3[] = {
     {"pagerline", P_NZINT, PI_TEXT, (void *)&PagerMax, CMT_PAGERLINE, NULL},
 #ifdef USE_HISTORY
+    {"use_history", P_INT, PI_ONOFF, (void *)&UseHistory, CMT_HISTORY, NULL},
     {"history", P_INT, PI_TEXT, (void *)&URLHistSize, CMT_HISTSIZE, NULL},
     {"save_hist", P_INT, PI_ONOFF, (void *)&SaveURLHist, CMT_SAVEHIST, NULL},
 #endif				/* USE_HISTORY */
@@ -628,14 +609,10 @@ struct param_ptr params3[] = {
 #ifdef USE_MARK
     {"mark", P_INT, PI_ONOFF, (void *)&use_mark, CMT_USE_MARK, NULL},
 #endif
-#ifdef EMACS_LIKE_LINEEDIT
     {"emacs_like_lineedit", P_INT, PI_ONOFF, (void *)&emacs_like_lineedit,
      CMT_EMACS_LIKE_LINEEDIT, NULL},
-#endif
-#ifdef VI_PREC_NUM
     {"vi_prec_num", P_INT, PI_ONOFF, (void *)&vi_prec_num, CMT_VI_PREC_NUM,
      NULL},
-#endif
     {"mark_all_pages", P_INT, PI_ONOFF, (void *)&MarkAllPages,
      CMT_MARK_ALL_PAGES, NULL},
     {"wrap_search", P_INT, PI_ONOFF, (void *)&WrapDefault, CMT_WRAP, NULL},
@@ -766,10 +743,8 @@ struct param_ptr params9[] = {
      (void *)&disable_secret_security_check, CMT_DISABLE_SECRET_SECURITY_CHECK,
      NULL},
     {"ftppasswd", P_STRING, PI_TEXT, (void *)&ftppasswd, CMT_FTPPASS, NULL},
-#ifdef FTPPASS_HOSTNAMEGEN
     {"ftppass_hostnamegen", P_INT, PI_ONOFF, (void *)&ftppass_hostnamegen,
      CMT_FTPPASS_HOSTNAMEGEN, NULL},
-#endif
     {"pre_form_file", P_STRING, PI_TEXT, (void *)&pre_form_file,
      CMT_PRE_FORM_FILE, NULL},
     {"user_agent", P_STRING, PI_TEXT, (void *)&UserAgent, CMT_USERAGENT, NULL},
@@ -1424,6 +1399,10 @@ init_rc(void)
 	interpret_rc(f);
 	fclose(f);
     }
+    if ((f = fopen(confFile(CONFIG_FILE), "rt")) != NULL) {
+	interpret_rc(f);
+	fclose(f);
+    }
     if ((f = fopen(config_file, "rt")) != NULL) {
 	interpret_rc(f);
 	fclose(f);
@@ -1485,7 +1464,7 @@ Buffer *
 load_option_panel(void)
 {
     Str src = Sprintf(optionpanel_src1, html_quote(w3m_version),
-		      html_quote(Local_cookie->ptr), CMT_HELPER);
+		      html_quote(localCookie()->ptr), CMT_HELPER);
     struct param_ptr *p;
     struct sel_c *s;
     int x, i;
@@ -1611,6 +1590,12 @@ etcFile(char *base)
     return expandPath(Strnew_m_charp(w3m_etc_dir(), "/", base, NULL)->ptr);
 }
 
+char *
+confFile(char *base)
+{
+    return expandPath(Strnew_m_charp(w3m_conf_dir(), "/", base, NULL)->ptr);
+}
+
 #ifndef USE_HELP_CGI
 char *
 helpFile(char *base)
