@@ -1,4 +1,4 @@
-/* $Id: table.c,v 1.40 2003/02/20 15:39:22 ukai Exp $ */
+/* $Id: table.c,v 1.46 2003/07/13 16:13:29 ukai Exp $ */
 /* 
  * HTML table
  */
@@ -2799,12 +2799,14 @@ feed_table_tag(struct table *tbl, char *line, struct table_mode *mode,
 	return TAG_ACTION_NONE;
     case HTML_P:
     case HTML_BR:
-    case HTML_DT:
-    case HTML_DD:
     case HTML_CENTER:
     case HTML_N_CENTER:
     case HTML_DIV:
     case HTML_N_DIV:
+	if (!(tbl->flag & TBL_IN_ROW))
+	    break;
+    case HTML_DT:
+    case HTML_DD:
     case HTML_H:
     case HTML_N_H:
     case HTML_LI:
@@ -2852,9 +2854,11 @@ feed_table_tag(struct table *tbl, char *line, struct table_mode *mode,
     case HTML_N_UL:
 	feed_table_block_tag(tbl, line, mode, -1, cmd);
 	break;
-    case HTML_PRE_INT:
     case HTML_NOBR:
     case HTML_WBR:
+	if (!(tbl->flag & TBL_IN_ROW))
+	    break;
+    case HTML_PRE_INT:
 	feed_table_inline_tag(tbl, line, mode, -1);
 	switch (cmd) {
 	case HTML_NOBR:
@@ -2877,6 +2881,8 @@ feed_table_tag(struct table *tbl, char *line, struct table_mode *mode,
 	}
 	break;
     case HTML_N_NOBR:
+	if (!(tbl->flag & TBL_IN_ROW))
+	    break;
 	feed_table_inline_tag(tbl, line, mode, -1);
 	if (mode->nobr_level > 0)
 	    mode->nobr_level--;
@@ -2888,6 +2894,7 @@ feed_table_tag(struct table *tbl, char *line, struct table_mode *mode,
 	mode->pre_mode &= ~TBLM_PRE_INT;
 	break;
     case HTML_IMG:
+	check_rowcol(tbl, mode);
 	w = tbl->fixed_width[tbl->col];
 	if (w < 0) {
 	    if (tbl->total_width > 0)
@@ -3490,6 +3497,89 @@ set_table_matrix0(struct table *t, int maxwidth)
     }
 }
 
+void
+check_relative_width(struct table *t, int maxwidth)
+{
+    int i;
+    double rel_total = 0;
+    int size = t->maxcol + 1;
+    double *rcolwidth = New_N(double, size);
+    struct table_cell *cell = &t->cell;
+    int n_leftcol = 0;
+
+    for (i = 0; i < size; i++)
+	rcolwidth[i] = 0;
+
+    for (i = 0; i < size; i++) {
+	if (t->fixed_width[i] < 0)
+	    rcolwidth[i] = -(double)t->fixed_width[i] / 100.0;
+	else if (t->fixed_width[i] > 0)
+	    rcolwidth[i] = (double)t->fixed_width[i] / maxwidth;
+	else
+	    n_leftcol++;
+    }
+    for (i = 0; i <= cell->maxcell; i++) {
+	if (cell->fixed_width[i] < 0) {
+	    double w = -(double)cell->fixed_width[i] / 100.0;
+	    double r;
+	    int j, k;
+	    int n_leftcell = 0;
+	    k = cell->col[i];
+	    r = 0.0;
+	    for (j = 0; j < cell->colspan[i]; j++) {
+		if (rcolwidth[j + k] > 0)
+		    r += rcolwidth[j + k];
+		else
+		    n_leftcell++;
+	    }
+	    if (n_leftcell == 0) {
+		/* w must be identical to r */
+		if (w != r)
+		    cell->fixed_width[i] = -100 * r;
+	    }
+	    else {
+		if (w <= r) {
+		    /* make room for the left(width-unspecified) cell */
+		    /* the next formula is an estimation of required width */
+		    w = r * cell->colspan[i] / (cell->colspan[i] - n_leftcell);
+		    cell->fixed_width[i] = -100 * w;
+		}
+		for (j = 0; j < cell->colspan[i]; j++) {
+		    if (rcolwidth[j + k] == 0)
+			rcolwidth[j + k] = (w - r) / n_leftcell;
+		}
+	    }
+	}
+	else if (cell->fixed_width[i] > 0) {
+	    /* todo */
+	}
+    }
+    /* sanity check */
+    for (i = 0; i < size; i++)
+	rel_total += rcolwidth[i];
+
+    if ((n_leftcol == 0 && rel_total < 0.9) || 1.1 < rel_total) {
+	for (i = 0; i < size; i++) {
+	    rcolwidth[i] /= rel_total;
+	}
+	for (i = 0; i < size; i++) {
+	    if (t->fixed_width[i] < 0)
+		t->fixed_width[i] = -rcolwidth[i] * 100;
+	}
+	for (i = 0; i <= cell->maxcell; i++) {
+	    if (cell->fixed_width[i] < 0) {
+		double r;
+		int j, k;
+		k = cell->col[i];
+		r = 0.0;
+		for (j = 0; j < cell->colspan[i]; j++)
+		    r += rcolwidth[j + k];
+		cell->fixed_width[i] = -r * 100;
+	    }
+	}
+    }
+}
+
 void
 set_table_matrix(struct table *t, int width)
 {
@@ -3510,6 +3600,8 @@ set_table_matrix(struct table *t, int width)
 	v_set_val(t->vector, i, 0.);
     }
 
+    check_relative_width(t, width);
+
     for (i = 0; i < size; i++) {
 	if (t->fixed_width[i] > 0) {
 	    a = max(t->fixed_width[i], t->minimum_width[i]);
