@@ -1,4 +1,4 @@
-/* $Id: url.c,v 1.54 2002/11/04 08:47:38 ukai Exp $ */
+/* $Id: url.c,v 1.82 2003/09/22 17:37:41 ukai Exp $ */
 #include "fm.h"
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -33,10 +33,14 @@
 
 #ifdef INET6
 /* see rc.c, "dns_order" and dnsorders[] */
-int ai_family_order_table[3][3] = {
+int ai_family_order_table[7][3] = {
     {PF_UNSPEC, PF_UNSPEC, PF_UNSPEC},	/* 0:unspec */
     {PF_INET, PF_INET6, PF_UNSPEC},	/* 1:inet inet6 */
-    {PF_INET6, PF_INET, PF_UNSPEC}	/* 2:inet6 inet */
+    {PF_INET6, PF_INET, PF_UNSPEC},	/* 2:inet6 inet */
+    {PF_UNSPEC, PF_UNSPEC, PF_UNSPEC},  /* 3: --- */
+    {PF_INET, PF_UNSPEC, PF_UNSPEC},    /* 4:inet */
+    {PF_UNSPEC, PF_UNSPEC, PF_UNSPEC},  /* 5: --- */
+    {PF_INET6, PF_UNSPEC, PF_UNSPEC},   /* 6:inet6 */
 };
 #endif				/* INET6 */
 
@@ -53,7 +57,10 @@ static int
     0,				/* local-CGI - not defined? */
     0,				/* exec - not defined? */
     119,			/* nntp */
+    119,			/* nntp group */
     119,			/* news */
+    119,			/* news group */
+    0,				/* data - not defined */
     0,				/* mailto - not defined */
 #ifdef USE_SSL
     443,			/* https */
@@ -68,7 +75,10 @@ struct cmdtable schemetable[] = {
     {"file", SCM_LOCAL},
     /*  {"exec", SCM_EXEC}, */
     {"nntp", SCM_NNTP},
+    /*  {"nntp", SCM_NNTP_GROUP}, */
     {"news", SCM_NEWS},
+    /*  {"news", SCM_NEWS_GROUP}, */
+    {"data", SCM_DATA},
 #ifndef USE_W3MMAILER
     {"mailto", SCM_MAILTO},
 #endif
@@ -80,23 +90,14 @@ struct cmdtable schemetable[] = {
 
 static struct table2 DefaultGuess[] = {
     {"html", "text/html"},
-    {"HTML", "text/html"},
     {"htm", "text/html"},
-    {"HTM", "text/html"},
     {"shtml", "text/html"},
-    {"SHTML", "text/html"},
     {"gif", "image/gif"},
-    {"GIF", "image/gif"},
     {"jpeg", "image/jpeg"},
     {"jpg", "image/jpeg"},
-    {"JPEG", "image/jpeg"},
-    {"JPG", "image/jpeg"},
     {"png", "image/png"},
-    {"PNG", "image/png"},
     {"xbm", "image/xbm"},
-    {"XBM", "image/xbm"},
     {"au", "audio/basic"},
-    {"AU", "audio/basic"},
     {"gz", "application/x-gzip"},
     {"Z", "application/x-compress"},
     {"bz2", "application/x-bzip"},
@@ -104,7 +105,6 @@ static struct table2 DefaultGuess[] = {
     {"zip", "application/x-zip"},
     {"lha", "application/x-lha"},
     {"lzh", "application/x-lha"},
-    {"LZH", "application/x-lha"},
     {"ps", "application/postscript"},
     {"pdf", "application/pdf"},
     {NULL, NULL}
@@ -148,7 +148,7 @@ loadMimeTypes(char *filename)
     Str tmp;
     struct table2 *mtypes;
 
-    f = fopen(expandName(filename), "r");
+    f = fopen(expandPath(filename), "r");
     if (f == NULL)
 	return NULL;
     n = 0;
@@ -222,6 +222,7 @@ DefaultFile(int scheme)
     case SCM_LOCAL:
     case SCM_LOCAL_CGI:
     case SCM_FTP:
+    case SCM_FTPDIR:
 	return allocStr("/", -1);
     }
     return NULL;
@@ -231,6 +232,7 @@ static MySignalHandler
 KeyAbort(SIGNAL_ARG)
 {
     LONGJMP(AbortLoading, 1);
+    SIGNAL_RETURN;
 }
 
 #ifdef USE_SSL
@@ -449,7 +451,7 @@ openSocket(char *const hostname,
     int a1, a2, a3, a4;
     unsigned long adr;
 #endif				/* not INET6 */
-    MySignalHandler(*volatile trap) (SIGNAL_ARG) = NULL;
+    MySignalHandler(*volatile prevtrap) (SIGNAL_ARG) = NULL;
 
     if (fmInitialized) {
 	message(Sprintf("Opening socket...")->ptr, 0, 0);
@@ -463,9 +465,7 @@ openSocket(char *const hostname,
 	    close(sock);
 	goto error;
     }
-    trap = signal(SIGINT, KeyAbort);
-    if (fmInitialized)
-	term_cbreak();
+    TRAP_ON;
     if (hostname == NULL) {
 #ifdef SOCK_DEBUG
 	sock_log("openSocket() failed. reason: Bad hostname \"%s\"\n",
@@ -610,15 +610,10 @@ openSocket(char *const hostname,
     }
 #endif				/* not INET6 */
 
-    if (fmInitialized)
-	term_raw();
-    signal(SIGINT, trap);
+    TRAP_OFF;
     return sock;
   error:
-    if (fmInitialized)
-	term_raw();
-    signal(SIGINT, trap);
-
+    TRAP_OFF;
     return -1;
 
 }
@@ -674,7 +669,10 @@ parseURL(char *url, ParsedURL *p_url, ParsedURL *current)
     p_url->query = NULL;
     p_url->label = NULL;
 
-    if (*url == '#') {		/* label only */
+    /* RFC1808: Relative Uniform Resource Locators
+     * 4.  Resolving Relative URLs
+     */
+    if (*url == '\0' || *url == '#') {
 	if (current)
 	    copyParsedURL(p_url, current);
 	goto do_label;
@@ -700,12 +698,29 @@ parseURL(char *url, ParsedURL *p_url, ParsedURL *current)
 	 * denotes a filename (therefore the scheme is SCM_LOCAL).
 	 */
 	if (current) {
-	    copyParsedURL(p_url, current);
-	    if (p_url->scheme == SCM_LOCAL_CGI)
+	    switch (current->scheme) {
+	    case SCM_LOCAL:
+	    case SCM_LOCAL_CGI:
 		p_url->scheme = SCM_LOCAL;
-	    /* label part and query part don't inherit */
-	    p_url->label = NULL;
-	    p_url->query = NULL;
+		break;
+	    case SCM_FTP:
+	    case SCM_FTPDIR:
+		p_url->scheme = SCM_FTP;
+		break;
+#ifdef USE_NNTP
+	    case SCM_NNTP:
+	    case SCM_NNTP_GROUP:
+		p_url->scheme = SCM_NNTP;
+		break;
+	    case SCM_NEWS:
+	    case SCM_NEWS_GROUP:
+		p_url->scheme = SCM_NEWS;
+		break;
+#endif
+	    default:
+		p_url->scheme = current->scheme;
+		break;
+	    }
 	}
 	else
 	    p_url->scheme = SCM_LOCAL;
@@ -755,27 +770,17 @@ parseURL(char *url, ParsedURL *p_url, ParsedURL *current)
     /*          ^p is here  */
   analyze_url:
     q = p;
-    while (*p && strchr(":/@?#", *p) == NULL) {
 #ifdef INET6
-	if (*p == '[') {	/* rfc2732 compliance */
-	    char *p_colon = NULL;
-	    do {
-		p++;
-		if ((p_colon == NULL) && (*p == ':'))
-		    p_colon = p;
-	    } while (*p && (IS_ALNUM(*p) || *p == ':' || *p == '.'));
-	    if (*p == ']') {
-		p++;
-		break;
-	    }
-	    else if (p_colon) {
-		p = p_colon;
-		break;
-	    }
-	}
-#endif
+    if (*q == '[') {		/* rfc2732,rfc2373 compliance */
 	p++;
+	while (IS_XDIGIT(*p) || *p == ':' || *p == '.')
+	    p++;
+	if (*p != ']' || (*(p + 1) && strchr(":/?#", *(p + 1)) == NULL))
+	    p = q;
     }
+#endif
+    while (*p && strchr(":/@?#", *p) == NULL)
+	p++;
     switch (*p) {
     case ':':
 	/* scheme://user:pass@host or
@@ -815,7 +820,8 @@ parseURL(char *url, ParsedURL *p_url, ParsedURL *current)
   analyze_file:
 #ifndef SUPPORT_NETBIOS_SHARE
     if (p_url->scheme == SCM_LOCAL && p_url->user == NULL &&
-	p_url->host != NULL && strcmp(p_url->host, "localhost")) {
+	p_url->host != NULL && *p_url->host != '\0' &&
+	strcmp(p_url->host, "localhost")) {
 	/*
 	 * In the environments other than CYGWIN, a URL like 
 	 * file://host/file is regarded as ftp://host/file.
@@ -830,6 +836,10 @@ parseURL(char *url, ParsedURL *p_url, ParsedURL *current)
 	    p_url->port = DefaultPort[SCM_FTP];
     }
 #endif
+    if ((*p == '\0' || *p == '#' || *p == '?') && p_url->host == NULL) {
+	p_url->file = "";
+	goto do_query;
+    }
 #ifdef SUPPORT_DOS_DRIVE_PREFIX
     if (p_url->scheme == SCM_LOCAL) {
 	q = p;
@@ -857,10 +867,9 @@ parseURL(char *url, ParsedURL *p_url, ParsedURL *current)
 #endif				/* USE_GOPHER */
     if (*p == '/')
 	p++;
-    if (*p == '\0') {		/* scheme://host[:port]/ */
+    if (*p == '\0' || *p == '#' || *p == '?') {	/* scheme://host[:port]/ */
 	p_url->file = DefaultFile(p_url->scheme);
-	p_url->label = NULL;
-	return;
+	goto do_query;
     }
 #ifdef USE_GOPHER
     if (p_url->scheme == SCM_GOPHER && *p == 'R') {
@@ -912,6 +921,7 @@ parseURL(char *url, ParsedURL *p_url, ParsedURL *current)
 	    p_url->file = copyPath(q, p - q, COPYPATH_SPC_IGNORE);
     }
 
+  do_query:
     if (*p == '?') {
 	q = ++p;
 	while (*p && *p != '#')
@@ -960,32 +970,67 @@ parseURL2(char *url, ParsedURL *pu, ParsedURL *current)
     if (pu->scheme == SCM_MAILTO)
 	return;
 #endif
+    if (pu->scheme == SCM_DATA)
+	return;
+    if (pu->scheme == SCM_NEWS || pu->scheme == SCM_NEWS_GROUP) {
+	if (pu->file && !strchr(pu->file, '@') &&
+	    (!(p = strchr(pu->file, '/')) || strchr(p + 1, '-') ||
+	     *(p + 1) == '\0'))
+	    pu->scheme = SCM_NEWS_GROUP;
+	else
+	    pu->scheme = SCM_NEWS;
+	return;
+    }
+    if (pu->scheme == SCM_NNTP || pu->scheme == SCM_NNTP_GROUP) {
+	if (pu->file && *pu->file == '/')
+	    pu->file = allocStr(pu->file + 1, -1);
+	if (pu->file && !strchr(pu->file, '@') &&
+	    (!(p = strchr(pu->file, '/')) || strchr(p + 1, '-') ||
+	     *(p + 1) == '\0'))
+	    pu->scheme = SCM_NNTP_GROUP;
+	else
+	    pu->scheme = SCM_NNTP;
+	if (current && (current->scheme == SCM_NNTP ||
+			current->scheme == SCM_NNTP_GROUP)) {
+	    if (pu->host == NULL) {
+		pu->host = current->host;
+		pu->port = current->port;
+	    }
+	}
+	return;
+    }
     if (pu->scheme == SCM_LOCAL)
-	pu->file = expandName(pu->file);
+	pu->file = file_quote(expandName(file_unquote(pu->file)));
 
-    if (current && pu->scheme == current->scheme) {
+    if (current && (pu->scheme == current->scheme ||
+		    (pu->scheme == SCM_FTP && current->scheme == SCM_FTPDIR) ||
+		    (pu->scheme == SCM_LOCAL &&
+		     current->scheme == SCM_LOCAL_CGI))
+	&& pu->host == NULL) {
 	/* Copy omitted element from the current URL */
-	if (pu->user == NULL) {
-	    pu->user = current->user;
-	}
-	if (pu->pass == NULL) {
-	    pu->pass = current->pass;
-	}
-	if (pu->host == NULL) {
-	    pu->host = current->host;
-	}
-	if (pu->file) {
-	    if (
+	pu->user = current->user;
+	pu->pass = current->pass;
+	pu->host = current->host;
+	pu->port = current->port;
+	if (pu->file && *pu->file) {
+#ifdef USE_EXTERNAL_URI_LOADER
+	    if (pu->scheme == SCM_UNKNOWN
+		&& strchr(pu->file, ':') == NULL
+		&& current && (p = strchr(current->file, ':')) != NULL) {
+		pu->file = Sprintf("%s:%s",
+				   allocStr(current->file,
+					    p - current->file), pu->file)->ptr;
+	    }
+	    else
+#endif
+		if (
 #ifdef USE_GOPHER
-		   pu->scheme != SCM_GOPHER &&
+		       pu->scheme != SCM_GOPHER &&
 #endif				/* USE_GOPHER */
-#ifdef USE_NNTP
-		   pu->scheme != SCM_NEWS &&
-#endif				/* USE_NNTP */
-		   pu->file[0] != '/'
+		       pu->file[0] != '/'
 #ifdef SUPPORT_DOS_DRIVE_PREFIX
-		   && !(pu->scheme == SCM_LOCAL && IS_ALPHA(pu->file[0])
-			&& pu->file[1] == ':')
+		       && !(pu->scheme == SCM_LOCAL && IS_ALPHA(pu->file[0])
+			    && pu->file[1] == ':')
 #endif
 		) {
 		/* file is relative [process 1] */
@@ -1009,9 +1054,10 @@ parseURL2(char *url, ParsedURL *pu, ParsedURL *current)
 	    }
 #endif				/* USE_GOPHER */
 	}
-	else if (pu->label) {
-	    /* pu has only label */
+	else {			/* scheme:[?query][#label] */
 	    pu->file = current->file;
+	    if (!pu->query)
+		pu->query = current->query;
 	}
 	/* comment: query part need not to be completed
 	 * from the current URL. */
@@ -1060,9 +1106,6 @@ parseURL2(char *url, ParsedURL *pu, ParsedURL *current)
 #ifdef USE_GOPHER
 		    pu->scheme != SCM_GOPHER &&
 #endif				/* USE_GOPHER */
-#ifdef USE_NNTP
-		    pu->scheme != SCM_NEWS &&
-#endif				/* USE_NNTP */
 		    pu->file[0] == '/') {
 	    /*
 	     * this happens on the following conditions:
@@ -1092,8 +1135,8 @@ _parsedURL2Str(ParsedURL *pu, int pass)
 {
     Str tmp;
     static char *scheme_str[] = {
-	"http", "gopher", "ftp", "ftp", "file", "file", "exec", "nntp", "news",
-	"mailto",
+	"http", "gopher", "ftp", "ftp", "file", "file", "exec", "nntp", "nntp",
+	"news", "news", "data", "mailto",
 #ifdef USE_SSL
 	"https",
 #endif				/* USE_SSL */
@@ -1129,8 +1172,12 @@ _parsedURL2Str(ParsedURL *pu, int pass)
 	return tmp;
     }
 #endif
+    if (pu->scheme == SCM_DATA) {
+	Strcat_charp(tmp, pu->file);
+	return tmp;
+    }
 #ifdef USE_NNTP
-    if (pu->scheme != SCM_NEWS)
+    if (pu->scheme != SCM_NEWS && pu->scheme != SCM_NEWS_GROUP)
 #endif				/* USE_NNTP */
     {
 	Strcat_charp(tmp, "//");
@@ -1152,7 +1199,7 @@ _parsedURL2Str(ParsedURL *pu, int pass)
     }
     if (
 #ifdef USE_NNTP
-	   pu->scheme != SCM_NEWS &&
+	   pu->scheme != SCM_NEWS && pu->scheme != SCM_NEWS_GROUP &&
 #endif				/* USE_NNTP */
 	   (pu->file == NULL || (pu->file[0] != '/'
 #ifdef SUPPORT_DOS_DRIVE_PREFIX
@@ -1236,13 +1283,20 @@ otherinfo(ParsedURL *target, ParsedURL *current, char *referer)
 	if (referer == NULL && current && current->scheme != SCM_LOCAL &&
 	    (current->scheme != SCM_FTP ||
 	     (current->user == NULL && current->pass == NULL))) {
+	    char *p = current->label;
 	    Strcat_charp(s, "Referer: ");
+	    current->label = NULL;
 	    Strcat(s, parsedURL2Str(current));
+	    current->label = p;
 	    Strcat_charp(s, "\r\n");
 	}
 	else if (referer != NULL && referer != NO_REFERER) {
+	    char *p = strchr(referer, '#');
 	    Strcat_charp(s, "Referer: ");
-	    Strcat_charp(s, referer);
+	    if (p)
+		Strcat_charp_n(s, referer, p - referer);
+	    else
+		Strcat_charp(s, referer);
 	    Strcat_charp(s, "\r\n");
 	}
     }
@@ -1322,7 +1376,7 @@ HTTPrequest(ParsedURL *pu, ParsedURL *current, HRequest *hr, TextList *extra)
 		seen_proxy_auth = 1;
 #ifdef USE_SSL
 		if (pu->scheme == SCM_HTTPS
-			&& hr->command != HR_COMMAND_CONNECT)
+		    && hr->command != HR_COMMAND_CONNECT)
 		    continue;
 #endif
 	    }
@@ -1331,9 +1385,9 @@ HTTPrequest(ParsedURL *pu, ParsedURL *current, HRequest *hr, TextList *extra)
 
     if (!seen_www_auth
 #ifdef USE_SSL
-	    && hr->command != HR_COMMAND_CONNECT
+	&& hr->command != HR_COMMAND_CONNECT
 #endif
-	    ) {
+	) {
 	Str auth_cookie = find_auth_cookie(pu->host, pu->port, pu->file, NULL);
 	if (auth_cookie)
 	    Strcat_m_charp(tmp, "Authorization: ", auth_cookie->ptr,
@@ -1342,13 +1396,13 @@ HTTPrequest(ParsedURL *pu, ParsedURL *current, HRequest *hr, TextList *extra)
 
     if (!seen_proxy_auth && (hr->flag & HR_FLAG_PROXY)
 #ifdef USE_SSL
-	    && (pu->scheme != SCM_HTTPS
-		|| hr->command == HR_COMMAND_CONNECT)
+	&& (pu->scheme != SCM_HTTPS || hr->command == HR_COMMAND_CONNECT)
 #endif
-	    ) {
+	) {
 	ParsedURL *proxy_pu = schemeToProxy(pu->scheme);
-	Str auth_cookie = find_auth_cookie(
-		proxy_pu->host, proxy_pu->port, proxy_pu->file, NULL);
+	Str auth_cookie =
+	    find_auth_cookie(proxy_pu->host, proxy_pu->port, proxy_pu->file,
+			     NULL);
 	if (!auth_cookie && proxy_auth_cookie)
 	    auth_cookie = proxy_auth_cookie;
 	if (auth_cookie)
@@ -1404,6 +1458,7 @@ HTTPrequest(ParsedURL *pu, ParsedURL *current, HRequest *hr, TextList *extra)
 void
 init_stream(URLFile *uf, int scheme, InputStream stream)
 {
+    memset(uf, 0, sizeof(URLFile));
     uf->stream = stream;
     uf->scheme = scheme;
     uf->encoding = ENC_7BIT;
@@ -1411,12 +1466,7 @@ init_stream(URLFile *uf, int scheme, InputStream stream)
     uf->compression = 0;
     uf->guess_type = NULL;
     uf->ext = NULL;
-}
-
-static InputStream
-openFTPStream(ParsedURL *pu)
-{
-    return newFileStream(openFTP(pu), closeFTP);
+    uf->modtime = -1;
 }
 
 URLFile
@@ -1425,19 +1475,13 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 	URLFile *ouf, HRequest *hr, unsigned char *status)
 {
     Str tmp;
-    int i, sock, scheme;
+    int sock, scheme;
     char *p, *q, *u;
     URLFile uf;
     HRequest hr0;
 #ifdef USE_SSL
     SSL *sslh = NULL;
 #endif				/* USE_SSL */
-#ifdef USE_NNTP
-    FILE *fw;
-    char *r;
-    InputStream stream;
-#endif				/* USE_NNTP */
-    int extlen = strlen(CGI_EXTENSION);
 
     if (hr == NULL)
 	hr = &hr0;
@@ -1476,6 +1520,7 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
     }
 
     uf.scheme = pu->scheme;
+    uf.url = parsedURL2Str(pu)->ptr;
     pu->is_nocache = (option->flag & RG_NOCACHE);
     uf.ext = filename_extension(pu->file, 1);
 
@@ -1487,49 +1532,22 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
     switch (pu->scheme) {
     case SCM_LOCAL:
     case SCM_LOCAL_CGI:
-	if (request && request->body) {
+	if (request && request->body)
 	    /* local CGI: POST */
-	    if ((q = strchr(pu->file, '?')) != NULL) {
-		p = Strnew_charp_n(pu->file, (int)(q - pu->file))->ptr;
-		pu->real_file = cleanupName(file_unquote(p));
-		q++;
-	    }
-	    uf.stream = newFileStream(localcgi_post(pu->real_file,
-						    pu->query,
-						    request,
-						    option->referer),
+	    uf.stream = newFileStream(localcgi_post(pu->real_file, pu->query,
+						    request, option->referer),
 				      (void (*)())pclose);
-	    if (uf.stream == NULL)
-		goto ordinary_local_file;
-	    uf.is_cgi = TRUE;
-	    uf.scheme = pu->scheme = SCM_LOCAL_CGI;
-	}
-	else if (pu->query != NULL) {
+	else
 	    /* lodal CGI: GET */
 	    uf.stream = newFileStream(localcgi_get(pu->real_file, pu->query,
 						   option->referer),
 				      (void (*)())pclose);
-	    if (uf.stream == NULL) {
-		goto ordinary_local_file;
-	    }
+	if (uf.stream) {
 	    uf.is_cgi = TRUE;
 	    uf.scheme = pu->scheme = SCM_LOCAL_CGI;
+	    return uf;
 	}
-	else if ((i = strlen(pu->file)) > extlen &&
-		 !strncmp(pu->file + i - extlen, CGI_EXTENSION, extlen)) {
-	    /* lodal CGI: GET */
-	    uf.stream = newFileStream(localcgi_get(pu->real_file, NULL,
-						   option->referer),
-				      (void (*)())pclose);
-	    if (uf.stream == NULL)
-		goto ordinary_local_file;
-	    uf.is_cgi = TRUE;
-	    uf.scheme = pu->scheme = SCM_LOCAL_CGI;
-	}
-	else {
-	  ordinary_local_file:
-	    examineFile(pu->real_file, &uf);
-	}
+	examineFile(pu->real_file, &uf);
 	if (uf.stream == NULL) {
 	    if (dir_exist(pu->real_file)) {
 		add_index_file(pu, &uf);
@@ -1569,6 +1587,7 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 	}
 	return uf;
     case SCM_FTP:
+    case SCM_FTPDIR:
 	if (pu->file == NULL)
 	    pu->file = allocStr("/", -1);
 	if (non_null(FTP_proxy) &&
@@ -1585,7 +1604,7 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 	    write(sock, tmp->ptr, tmp->length);
 	}
 	else {
-	    uf.stream = openFTPStream(pu);
+	    uf.stream = openFTPStream(pu, &uf);
 	    uf.scheme = pu->scheme;
 	    return uf;
 	}
@@ -1600,8 +1619,11 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 	    hr->command = HR_COMMAND_POST;
 	if (request && request->method == FORM_METHOD_HEAD)
 	    hr->command = HR_COMMAND_HEAD;
-	if (non_null(HTTP_proxy) &&
-	    !Do_not_use_proxy &&
+	if ((
+#ifdef USE_SSL
+		(pu->scheme == SCM_HTTPS) ? non_null(HTTPS_proxy) :
+#endif				/* USE_SSL */
+		non_null(HTTP_proxy)) && !Do_not_use_proxy &&
 	    pu->host != NULL && !check_no_proxy(pu->host)) {
 	    char *save_label;
 	    hr->flag |= HR_FLAG_PROXY;
@@ -1609,22 +1631,26 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 	    if (pu->scheme == SCM_HTTPS && *status == HTST_CONNECT) {
 		sock = ssl_socket_of(ouf->stream);
 		if (!(sslh = openSSLHandle(sock, pu->host,
-				&uf.ssl_certificate))) {
+					   &uf.ssl_certificate))) {
 		    *status = HTST_MISSING;
 		    return uf;
 		}
 	    }
+	    else if (pu->scheme == SCM_HTTPS) {
+		sock = openSocket(HTTPS_proxy_parsed.host,
+				  schemetable[HTTPS_proxy_parsed.scheme].
+				  cmdname, HTTPS_proxy_parsed.port);
+		sslh = NULL;
+	    }
 	    else {
+#endif				/* USE_SSL */
 		sock = openSocket(HTTP_proxy_parsed.host,
 				  schemetable[HTTP_proxy_parsed.scheme].
 				  cmdname, HTTP_proxy_parsed.port);
+#ifdef USE_SSL
 		sslh = NULL;
 	    }
-#else
-	    sock = openSocket(HTTP_proxy_parsed.host,
-			      schemetable[HTTP_proxy_parsed.scheme].cmdname,
-			      HTTP_proxy_parsed.port);
-#endif
+#endif				/* USE_SSL */
 	    if (sock < 0) {
 #ifdef SOCK_DEBUG
 		sock_log("Can't open socket\n");
@@ -1664,7 +1690,7 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 #ifdef USE_SSL
 	    if (pu->scheme == SCM_HTTPS) {
 		if (!(sslh = openSSLHandle(sock, pu->host,
-				&uf.ssl_certificate))) {
+					   &uf.ssl_certificate))) {
 		    *status = HTST_MISSING;
 		    return uf;
 		}
@@ -1738,7 +1764,7 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 		return uf;
 	    if (pu->file == NULL)
 		pu->file = "1";
-	    tmp = Strnew_charp(url_unquote(pu->file));
+	    tmp = Strnew_charp(file_unquote(pu->file));
 	    Strcat_char(tmp, '\n');
 	}
 	write(sock, tmp->ptr, tmp->length);
@@ -1746,84 +1772,35 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 #endif				/* USE_GOPHER */
 #ifdef USE_NNTP
     case SCM_NNTP:
-	/* nntp://<host>:<port>/<newsgroup-name>/<article-number> */
+    case SCM_NNTP_GROUP:
     case SCM_NEWS:
-        /* news:<newsgroup-name> XXX: not yet */
-        /* news:<unique>@<full_domain_name> */
-	if (pu->scheme == SCM_NNTP) {
-	    p = pu->host;
-	}
-	else {
-	    p = getenv("NNTPSERVER");
-	}
-	r = getenv("NNTPMODE");
-	if (p == NULL)
-	    return uf;
-	sock = openSocket(p, "nntp", pu->port);
-	if (sock < 0)
-	    return uf;
-	stream = newInputStream(sock);
-	fw = fdopen(sock, "wb");
-	if (stream == NULL || fw == NULL)
-	    return uf;
-	tmp = StrISgets(stream);
-	if (tmp->length == 0)
-	    goto nntp_error;
-	sscanf(tmp->ptr, "%d", &i);
-	if (i != 200 && i != 201)
-	    goto nntp_error;
-	if (r && *r != '\0') {
-	    fprintf(fw, "MODE %s\r\n", r);
-	    fflush(fw);
-	    tmp = StrISgets(stream);
-	    if (tmp->length == 0)
-		goto nntp_error;
-	    sscanf(tmp->ptr, "%d", &i);
-	    if (i != 200 && i != 201)
-		goto nntp_error;
-	}
-	if (pu->scheme == SCM_NNTP) {
-	    char *group;
-	    if (pu->file == NULL || *pu->file == '\0')
-		goto nntp_error;
-	    /* first char of pu->file is '/' */
-	    group = url_unquote(Strnew_charp(pu->file + 1)->ptr);
-	    p = strchr(group, '/');
-	    if (p == NULL)
-		goto nntp_error;
-	    *p++ = '\0';
-	    fprintf(fw, "GROUP %s\r\n", group);
-	    fflush(fw);
-	    tmp = StrISgets(stream);
-	    if (tmp->length == 0) {
-		goto nntp_error;
-	    }
-	    sscanf(tmp->ptr, "%d", &i);
-	    if (i != 211)
-		goto nntp_error;
-	    fprintf(fw, "ARTICLE %s\r\n", p);
-	}
-	else {
-	    if (pu->file == NULL || *pu->file == '\0')
-		goto nntp_error;
-	    /* pu-file contains '@' => news:<message-id> */
-	    fprintf(fw, "ARTICLE <%s>\r\n", url_unquote(pu->file));
-	}
-	fflush(fw);
-	tmp = StrISgets(stream);
-	if (tmp->length == 0)
-	    goto nntp_error;
-	sscanf(tmp->ptr, "%d", &i);
-	if (i != 220)
-	    goto nntp_error;
-	uf.scheme = SCM_NEWS;	/* XXX */
-	uf.stream = stream;
-	return uf;
-      nntp_error:
-	ISclose(stream);
-	fclose(fw);
+    case SCM_NEWS_GROUP:
+	if (pu->scheme == SCM_NNTP || pu->scheme == SCM_NEWS)
+	    uf.scheme = SCM_NEWS;
+	else
+	    uf.scheme = SCM_NEWS_GROUP;
+	uf.stream = openNewsStream(pu);
 	return uf;
 #endif				/* USE_NNTP */
+    case SCM_DATA:
+	if (pu->file == NULL)
+	    return uf;
+	p = Strnew_charp(pu->file)->ptr;
+	q = strchr(p, ',');
+	if (q == NULL)
+	    return uf;
+	*q++ = '\0';
+	tmp = Strnew_charp(q);
+	q = strrchr(p, ';');
+	if (q != NULL && !strcmp(q, ";base64")) {
+	    *q = '\0';
+	    uf.encoding = ENC_BASE64;
+	}
+	else
+	    tmp = Str_url_unquote(tmp, FALSE, FALSE);
+	uf.stream = newStrStream(tmp);
+	uf.guess_type = (*p != '\0') ? p : "text/plain";
+	return uf;
     case SCM_UNKNOWN:
     default:
 	return uf;
@@ -1856,6 +1833,7 @@ add_index_file(ParsedURL *pu, URLFile *uf)
 static char *
 guessContentTypeFromTable(struct table2 *table, char *filename)
 {
+    struct table2 *t;
     char *p;
     if (table == NULL)
 	return NULL;
@@ -1865,10 +1843,13 @@ guessContentTypeFromTable(struct table2 *table, char *filename)
     if (p == filename)
 	return NULL;
     p++;
-    while (table->item1) {
-	if (!strcmp(p, table->item1))
-	    return table->item2;
-	table++;
+    for (t = table; t->item1; t++) {
+	if (!strcmp(p, t->item1))
+	    return t->item2;
+    }
+    for (t = table; t->item1; t++) {
+	if (!strcasecmp(p, t->item1))
+	    return t->item2;
     }
     return NULL;
 }
@@ -1943,6 +1924,8 @@ int
 check_no_proxy(char *domain)
 {
     TextListItem *tl;
+    volatile int ret = 0;
+    MySignalHandler(*volatile prevtrap) (SIGNAL_ARG) = NULL;
 
     if (NO_proxy_domains == NULL || NO_proxy_domains->nitem == 0 ||
 	domain == NULL)
@@ -1957,6 +1940,11 @@ check_no_proxy(char *domain)
     /* 
      * to check noproxy by network addr
      */
+    if (SETJMP(AbortLoading) != 0) {
+	ret = 0;
+	goto end;
+    }
+    TRAP_ON;
     {
 #ifndef INET6
 	struct hostent *he;
@@ -1965,8 +1953,10 @@ check_no_proxy(char *domain)
 	char addr[4 * 16], buf[5];
 
 	he = gethostbyname(domain);
-	if (!he)
-	    return (0);
+	if (!he) {
+	    ret = 0;
+	    goto end;
+	}
 	for (h_addr_list = (unsigned char **)he->h_addr_list; *h_addr_list;
 	     h_addr_list++) {
 	    sprintf(addr, "%d", h_addr_list[0][0]);
@@ -1975,8 +1965,10 @@ check_no_proxy(char *domain)
 		strcat(addr, buf);
 	    }
 	    for (tl = NO_proxy_domains->first; tl != NULL; tl = tl->next) {
-		if (strncmp(tl->ptr, addr, strlen(tl->ptr)) == 0)
-		    return (1);
+		if (strncmp(tl->ptr, addr, strlen(tl->ptr)) == 0) {
+		    ret = 1;
+		    goto end;
+		}
 	    }
 	}
 #else				/* INET6 */
@@ -2016,7 +2008,8 @@ check_no_proxy(char *domain)
 		for (tl = NO_proxy_domains->first; tl != NULL; tl = tl->next) {
 		    if (strncmp(tl->ptr, addr, strlen(tl->ptr)) == 0) {
 			freeaddrinfo(res0);
-			return 1;
+			ret = 1;
+			goto end;
 		    }
 		}
 	    }
@@ -2027,7 +2020,9 @@ check_no_proxy(char *domain)
 	}
 #endif				/* INET6 */
     }
-    return 0;
+  end:
+    TRAP_OFF;
+    return ret;
 }
 
 char *
@@ -2074,7 +2069,7 @@ loadURIMethods(char *filename)
     struct table2 *um;
     char *up, *p;
 
-    f = fopen(expandName(filename), "r");
+    f = fopen(expandPath(filename), "r");
     if (f == NULL)
 	return NULL;
     i = 0;
@@ -2210,23 +2205,25 @@ schemeToProxy(int scheme)
 {
     ParsedURL *pu = NULL;	/* for gcc */
     switch (scheme) {
-	case SCM_HTTP:
+    case SCM_HTTP:
+	pu = &HTTP_proxy_parsed;
+	break;
 #ifdef USE_SSL
-	case SCM_HTTPS:
+    case SCM_HTTPS:
+	pu = &HTTPS_proxy_parsed;
+	break;
 #endif
-	    pu = &HTTP_proxy_parsed;
-	    break;
-	case SCM_FTP:
-	    pu = &FTP_proxy_parsed;
-	    break;
+    case SCM_FTP:
+	pu = &FTP_proxy_parsed;
+	break;
 #ifdef USE_GOPHER
-	case SCM_GOPHER:
-	    pu = &GOPHER_proxy_parsed;
-	    break;
+    case SCM_GOPHER:
+	pu = &GOPHER_proxy_parsed;
+	break;
 #endif
 #ifdef DEBUG
-	default:
-	    abort();
+    default:
+	abort();
 #endif
     }
     return pu;
